import click
from thunder import api
from thunder import auth_helper


def login():
    id_token, refresh_token, uid = api.authenticate_user()
    if id_token and refresh_token:
        auth_helper.save_tokens(id_token, refresh_token, uid)
        click.echo("Logged in successfully.")
    else:
        click.echo("Login failed.")
    return id_token, refresh_token, uid


def logout():
    auth_helper.delete_data()
    click.echo("Logged out successfully.")


def validate_auth():
    id_token, refresh_token, uid = auth_helper.load_tokens()
    # print(id_token)

    if id_token is None or not api.is_token_valid(id_token):
        if refresh_token:
            new_id_token, new_refresh_token, uid = api.refresh_id_token(refresh_token)
            if new_id_token and new_refresh_token:
                auth_helper.save_tokens(new_id_token, new_refresh_token, uid)
                return new_id_token, new_refresh_token, uid
        return None, None, None
    return id_token, refresh_token, uid



import requests
import click

FIREBASE_API_KEY = "AIzaSyDFojZdCcPvKAczWrZTl-6DMtSq8E2X3Xk"


def authenticate_user():
    email = click.prompt("Please enter your email", type=str)
    password = click.prompt("Please enter your password", type=str, hide_input=True)

    data = {
        "email": email,
        "password": password,
        "returnSecureToken": True,
    }

    response = requests.post(
        f"https://identitytoolkit.googleapis.com/v1/accounts:signInWithPassword?key={FIREBASE_API_KEY}",
        data=data,
    )

    if response.ok:
        id_token = response.json()["idToken"]
        refresh_token = response.json()["refreshToken"]
        uid = response.json()["localId"]

        return id_token, refresh_token, uid
    else:
        click.echo("Authentication failed.")
        return None, None, None


def refresh_id_token(refresh_token):
    data = {
        "grant_type": "refresh_token",
        "refresh_token": refresh_token,
    }

    response = requests.post(
        f"https://securetoken.googleapis.com/v1/token?key={FIREBASE_API_KEY}", data=data
    )

    if response.ok:
        id_token = response.json()["id_token"]
        new_refresh_token = response.json()["refresh_token"]
        uid = response.json()["user_id"]
        return id_token, new_refresh_token, uid
    else:
        return None, None, None


def is_token_valid(id_token):
    # Endpoint to verify the ID token
    url = f"https://identitytoolkit.googleapis.com/v1/accounts:lookup?key={FIREBASE_API_KEY}"

    data = {"idToken": id_token}

    response = requests.post(url, data=data)
    # If the response is successful, the token is valid
    return response.ok



import click
from thunder import auth
import sys
import os
from thunder.thunder_helper import Task
from importlib.metadata import version
import requests
from packaging import version as version_parser

package_name = "tnr"  # update if name changes
# Get the directory of the current file (thunder.py), then go up two levels to the root
current_dir = os.path.dirname(os.path.abspath(__file__))
root_dir = os.path.join(current_dir, "..", "..")

# Add the src directory to sys.path
# src_dir = os.path.join(root_dir, 'src')
sys.path.append(root_dir)


class DefaultCommandGroup(click.Group):
    def resolve_command(self, ctx, args):
        try:
            # Try to resolve the command normally
            check_for_update()
            return super(DefaultCommandGroup, self).resolve_command(ctx, args)
        except click.exceptions.UsageError:
            # If no command is found, default to 'run' and include the args
            return "run", run, args


@click.group(
    cls=DefaultCommandGroup,
    help="This CLI is the interface between you and the Thunder network.",
)
def cli():
    pass


@cli.command(
    help="Runs a specified task on the Thunder network. This is the default behavior of the thunder command. Please see thundergpu.net for detailed documentation."
)
@click.option("--ngpus", default=1, help="Specify the number of GPUs to use.")
@click.argument("args", nargs=-1)
def run(ngpus, args):
    """
    Executes a task on the Thunder network.

    Args:
        ngpus: The number of GPUs to use for the task.
        args: Additional arguments to pass to the task.
    """
    id_token, refresh_token, uid = auth.validate_auth()

    if not id_token or not refresh_token:
        click.echo("Please log in to begin using the Thunder network.")
        id_token, refresh_token, uid = auth.login()

        if not id_token or not refresh_token:
            click.echo("Authentication failed. Unable to proceed.")
            return

    # Create an instance of Task with required arguments
    task = Task(ngpus, args, uid)

    # Initialize the session with the current auth token
    if not task.get_password(id_token):
        click.echo("Failed to retrieve password for session.")
        return

    # Execute the task
    if not task.execute_task(id_token, args):
        click.echo("Failed to execute the task.")
        return

    # Close the session
    if not task.close_session(id_token):
        click.echo("Failed to close the session.")


@cli.command(help="Logs in to the Thunder network.")
def login():
    """
    Authenticates a user for access to the Thunder network. Please use the same login credentials as the Thunder console (console.thundergpu.net).
    """
    auth.login()


@cli.command(help="Logs out from the Thunder network.")
def logout():
    """
    Logs the current user out of the Thunder network.
    """
    auth.logout()


def check_for_update():
    try:
        current_version = version(package_name)
        response = requests.get(f"https://pypi.org/pypi/{package_name}/json", timeout=1)
        latest_version = response.json()["info"]["version"]
        if version_parser.parse(current_version) < version_parser.parse(latest_version):
            click.echo(
                f"Update available: {latest_version}. Run `pip install --upgrade {package_name}` to update.",
                err=True,
            )
    except Exception as e:
        click.echo(f"Error checking for update: {e}", err=True)


if __name__ == "__main__":
    cli()



import sys
import click
import requests
import base64
import os

sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
from thunder.get_latest import get_latest

# URLs for cloud functions
CREATE_SESSION_USER_URL = "https://create-session-user-mumhtj6swa-uc.a.run.app"
DELETE_SESSION_USER_URL = "https://delete-session-user-mumhtj6swa-uc.a.run.app"


# Helper to call cloud functions
def call_firebase_function(url, id_token, payload=None):
    headers = {
        "Authorization": "Bearer " + id_token,
        "Content-Type": "application/json",
    }

    response = requests.post(url, headers=headers, json=payload)

    if response.status_code == 200:
        return response
    else:
        return f"Error: {response.status_code}, {response.text}"


class Task:
    def __init__(self, ngpus, args, uid):
        self.ngpus = ngpus
        self.args = args
        self.username = uid
        self.password = None
        self.topic = None

    # set environment variable for
    # use firebase credentials to retrieve session password
    def get_password(self, id_token):
        payload = {}
        response = call_firebase_function(
            CREATE_SESSION_USER_URL, id_token, payload
        ).json()
        if not isinstance(response, dict):
            click.echo("Failed to create user.")
            return False

        password = response.get("password")

        if not password:
            click.echo("Invalid response: Password missing.")
            return False

        self.password = password
        return True

        # Decode the password from Base64 back to bytes
        # try:
        #     self.password =
        #     # print(password_encoded)
        #     # self.password = str(base64.b64decode(password_encoded))
        #     # print(self.password)
        #     return True
        # except Exception as e:
        #     click.echo(f"Failed to decode password: {e}")
        #     return False

    def execute_task(self, id_token, args):
        # to implement, communicate to manager that a task is beginning, triggering start_user
        # look up python os.fork process set environment variables
        click.echo("executing task")

        # We need to find the shared object
        # binary = "/thunder/client_linux_x86_64"
        # if not os.path.exists("/thunder/client_linux_x86_64"):
        #     click.echo("thunder.so file not found")
        #     return False

        # Get from https://storage.googleapis.com/thunder_binary/client
        binary = get_latest("client", "/root/thunder.so")
        if binary == None:
            print("Failed to download binary")
            return False

        os.environ["SESSION_USERNAME"] = self.username
        os.environ["SESSION_PASSWORD"] = self.password
        os.environ["TOKEN"] = id_token
        os.environ["LD_PRELOAD"] = binary

        # This should never return
        os.execvp(args[0], args)
        return False

    def close_session(self, id_token):
        # to implement, communicate to manager computation is done, triggering exit_user
        click.echo("finishing task")
        return True



from setuptools import setup, find_packages

setup(
    name="tnr",
    version="1.1.1",
    package_dir={"": "src"},  # Specify the root directory for packages
    packages=find_packages(where="src"),  # Tell setuptools to find packages under src
    include_package_data=True,  # Include other files specified in MANIFEST.in
    install_requires=["Click", "requests", "cryptography", "common"],
    entry_points={"console_scripts": ["thunder=thunder.thunder:cli"]},
)

# delete old dist folder first, and increment version number
# to build: python setup.py sdist bdist_wheel
# to distribute: twine upload dist/* --repository testpypi
# to install: pip install thunder-client



from cryptography.fernet import Fernet
import os
import click

KEY = b"Y9_k5HsxyFAjZ1QcwxeB36IAWVjQW-VVb6_quqaLWDQ="
encrypted = False


def encrypt_data(data):
    if encrypted:
        cipher_suite = Fernet(KEY)
        encrypted_data = cipher_suite.encrypt(data.encode())
        return encrypted_data.decode()
    else:
        return data


def decrypt_data(encrypted_data):
    if encrypted:
        cipher_suite = Fernet(KEY)
        decrypted_data = cipher_suite.decrypt(encrypted_data.encode())
        return decrypted_data.decode()
    else:
        return encrypted_data


def save_tokens(id_token, refresh_token, uid):
    with open(".thunder", "w") as file:
        file.write(encrypt_data(id_token) + "\n")
        file.write(encrypt_data(refresh_token) + "\n")
        file.write(uid)


def load_tokens():
    try:
        with open(".thunder", "r") as file:
            encrypted_id_token = file.readline().strip()
            encrypted_refresh_token = file.readline().strip()
            uid = file.readline().strip()
            if encrypted_id_token and encrypted_refresh_token:
                return (
                    decrypt_data(encrypted_id_token),
                    decrypt_data(encrypted_refresh_token),
                    uid,
                )
            else:
                click.echo("error loading tokens!")
                return None, None, None
    except:
        return None, None, None


def delete_data():
    try:
        os.remove(".thunder")
    except OSError:
        pass
